"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const rx = require("rxjs");
const error_1 = require("./error");
var error_2 = require("./error");
exports.Error = error_2.Error;
const log_msg_1 = require("./log-msg");
var log_msg_2 = require("./log-msg");
exports.LogMsg = log_msg_2.LogMsg;
var Match;
(function (Match) {
    Match["NUMBER"] = "number";
})(Match = exports.Match || (exports.Match = {}));
class Matcher {
    constructor(type, cb) {
        this.type = type;
        this.cb = cb;
    }
}
exports.Matcher = Matcher;
class Done {
    constructor(result) {
        this.result = result;
    }
}
exports.Done = Done;
class Complete {
}
exports.Complete = Complete;
class Subject extends rx.Subject {
    constructor() {
        super();
        this.obss = [];
        this.matcher = [
            new Matcher(null, (obs, _data) => {
                // console.log(`[${this.objectId}]:[${obs.objectId}]:wildcard:${JSON.stringify(_data)}`);
                obs.next(_data);
                return true;
            })
        ];
        this._completed = [];
        this.objectId = ('' + (1000000000 + ~~(Math.random() * 1000000000))).slice(1);
    }
    done(result) {
        this.next(done(result));
        return this;
    }
    searchMatcher(matcherIdx, obssIdx, rxme, found) {
        // console.log(`[${this.objectId}]:enter:${!!this._done}`);
        if (matcherIdx >= this.matcher.length) {
            this._completed.forEach(cd => cd(null, rxme));
            return;
        }
        const match = this.matcher[matcherIdx];
        if (rxme.isKind(match.type)) {
            if (obssIdx >= this.obss.length || (found && obssIdx == this.obss.length - 1)) {
                this.searchMatcher(matcherIdx + 1, 0, rxme, found);
                return;
            }
            const os = this.obss[obssIdx];
            const doneFilter = new Subject();
            const isCompleted = rxme.isKind(Complete);
            const doneFilterSubscription = doneFilter.subscribe(obs => {
                if (obs.data instanceof Done) {
                    doneFilterSubscription.unsubscribe();
                    // if (isCompleted && os) {
                    //   os.next(rxme);
                    //   // console.log('matchDone', this.objectId, os.objectId, isCompleted, rxme);
                    // }
                    this.searchMatcher(matcherIdx, obssIdx + 1, rxme, (found || obs.data.result) && !isCompleted);
                }
                else {
                    if (os) {
                        os.next(obs);
                    }
                }
            });
            // console.log(`[${this.objectId}]:preMatch:cb:${!!this._done}`);
            const o = match.cb(doneFilter, rxme.asKindAny(match.type));
            if (!(o instanceof Subject)) {
                doneFilter.done(o);
            }
        }
        else {
            this.searchMatcher(matcherIdx + 1, 0, rxme, found);
        }
        // console.log(`[${this.objectId}]:leave:${!!this._done}`);
    }
    passTo(obs = null) {
        let obss;
        if (obs instanceof Array) {
            obss = obs;
        }
        else {
            obss = [obs];
        }
        // if (!!obss.find(o => !o)) {
        // console.error(`[${this.objectId}]:NULL:obss`);
        // }
        this.obss.push.apply(this.obss, obss);
        // console.log(`[${obss}]:[${this.obss}]`);
        if (!this.subscription) {
            // console.log(`[${this.objectId}]:passTo:[${this.subscription}]`);
            this.subscription = this.subscribe(myobs => {
                // console.log(`[${this.objectId}]:Matcher:${JSON.stringify(myobs)}`);
                this.searchMatcher(0, 0, myobs, false);
            }, (err) => {
                this.searchMatcher(0, 0, error(err), false);
            }, () => {
                this.searchMatcher(0, 0, complete(), false);
            });
        }
        return this;
    }
    completed(cb) {
        this._completed.push(cb);
        return this;
    }
    wildCard(cb) {
        this.match(null, cb);
        return this;
    }
    match(typ, cb) {
        this.matcher.push(new Matcher(typ, cb));
        // this make the wildcard the last in match chain
        const swap = this.matcher[this.matcher.length - 1];
        this.matcher[this.matcher.length - 1] = this.matcher[this.matcher.length - 2];
        this.matcher[this.matcher.length - 2] = swap;
        // console.log('Match:push:', this.matcher.length);
        return this;
    }
    matchError(cb) {
        this.match(error_1.default, cb);
        return this;
    }
    matchLogMsg(cb) {
        this.match(log_msg_1.default, cb);
        return this;
    }
    matchDone(cb) {
        this.match(Done, cb);
        return this;
    }
    matchComplete(cb) {
        this.match(Complete, cb);
        return this;
    }
}
exports.Subject = Subject;
class Observable extends rx.Observable {
}
exports.Observable = Observable;
class RxMe {
    constructor(_data) {
        this.objectId = ('' + (1000000000 + ~~(Math.random() * 1000000000))).slice(1);
        this.data = _data;
    }
    passthrough(obs) {
        obs.next(this);
        return true;
    }
    forward(obs) {
        if (this.isError() || this.isLogMsg()) {
            obs.next(this);
            return true;
        }
        return false;
    }
    isLogMsg() {
        return this.isKind(log_msg_1.default);
    }
    asLogMsg() {
        if (!this.isLogMsg()) {
            throw 'could not convert to LogMesg';
        }
        return this.data;
    }
    isError() {
        return this.isKind(error_1.default);
    }
    asError() {
        if (!this.isError()) {
            throw 'could not convert to LogMesg';
        }
        return this.data;
    }
    isKind(a) {
        if (!a) {
            return true;
        }
        else if (typeof (a) == 'function') {
            return this.data instanceof a;
        }
        else {
            return typeof (this.data) == a;
        }
    }
    asKind() {
        return this.data;
    }
    asKindAny(a) {
        if (!a) {
            return this;
        }
        return this.data;
    }
}
exports.RxMe = RxMe;
function data(t) {
    return new RxMe(t);
}
exports.data = data;
function logMsg(level, ...parts) {
    return data(new log_msg_1.default(level, parts));
}
exports.logMsg = logMsg;
function error(a) {
    return data(new error_1.default(a));
}
exports.error = error;
function done(a) {
    return data(new Done(a));
}
exports.done = done;
function complete() {
    return data(new Complete());
}
exports.complete = complete;
exports.default = RxMe;
//# sourceMappingURL=rxme.js.map